@@ -60,10 +60,24 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
 #endif
   public:
 
+    //! Legend node data roles
     enum LegendNodeRoles
     {
-      RuleKeyRole = Qt::UserRole,     //!< Rule key of the node (QString)
-      ParentRuleKeyRole               //!< Rule key of the parent legend node - for legends with tree hierarchy (QString). Added in 2.8
+      RuleKeyRole = Qt::UserRole, //!< Rule key of the node (QString)
+      ParentRuleKeyRole, //!< Rule key of the parent legend node - for legends with tree hierarchy (QString). Added in 2.8
+      NodeTypeRole, //!< Type of node. Added in 3.16
+    };
+
+    //! Types of legend nodes
+    enum NodeTypes
+    {
+      SimpleLegend, //!< Simple label with icon legend node type
+      SymbolLegend, //!< Vector symbol legend node type
+      RasterSymbolLegend, //!< Raster symbol legend node type
+      ImageLegend, //!< Raster image legend node type
+      WmsLegend, //!< WMS legend node type
+      DataDefinedSizeLegend, //!< Marker symbol legend node type
+      EmbeddedWidget, //!< Embedded widget placeholder node type
     };
 
     //! Returns pointer to the parent layer node
@@ -107,7 +121,7 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
      * \see userPatchSize()
      * \since QGIS 3.14
      */
-    virtual void setUserPatchSize( QSizeF size ) { mUserSize = size; }
+    virtual void setUserPatchSize( QSizeF size );
 
     /**
      * Sets whether a forced column break should occur before the node.
@@ -129,7 +143,8 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
 
     /**
      * Notification from model that information from associated map view has changed.
-     *  Default implementation does nothing. */
+     *  Default implementation does nothing.
+    */
     virtual void invalidateMapBasedData() {}
 
     struct ItemContext
@@ -254,6 +269,13 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
     //! Emitted on internal data change so the layer tree model can forward the signal to views
     void dataChanged();
 
+    /**
+     * Emitted when the size of this node changes.
+     *
+     * \since QGIS 3.16
+     */
+    void sizeChanged();
+
   protected:
     //! Construct the node with pointer to its parent layer node
     explicit QgsLayerTreeModelLegendNode( QgsLayerTreeLayer *nodeL, QObject *parent SIP_TRANSFERTHIS = nullptr );
@@ -269,6 +291,7 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
     QSizeF mUserSize;
     bool mColumnBreakBeforeNode = false;
 };
+Q_DECLARE_METATYPE( QgsLayerTreeModelLegendNode::NodeTypes )
 
 #include "qgslegendsymbolitem.h"
 #include "qgstextformat.h"
@@ -464,7 +487,10 @@ class CORE_EXPORT QgsSymbolLegendNode : public QgsLayerTreeModelLegendNode
     mutable QPixmap mPixmap; // cached symbol preview
     QString mLabel;
     bool mSymbolUsesMapUnits;
+
     QSize mIconSize;
+    double mSymbolMinimumSize = 0.5;
+    double mSymbolMaximumSize = 20.0;
 
     QString mTextOnSymbolLabel;
     QgsTextFormat mTextOnSymbolTextFormat;
